@@ -47,7 +47,7 @@ type Chunk struct {
 
 type chunkerState struct {
 	window [windowSize]byte
-	wpos   int
+	wpos   uint
 
 	buf  []byte
 	bpos uint
@@ -222,9 +222,14 @@ func (c *Chunker) Next(data []byte) (Chunk, error) {
 		return Chunk{}, errors.New("tables for polynomial computation not initialized")
 	}
 
-	tabout := c.tables.out
-	tabmod := c.tables.mod
+	tab := &c.tables
 	polShift := c.polShift
+	// go guarantees the expected behavior for bit shifts even for shift counts
+	// larger than the value width. Bounding the value of polShift allows the compiler
+	// to optimize the code for 'digest >> polShift'
+	if polShift > 53-8 {
+		return Chunk{}, errors.New("the polynomial must have a degree less than or equal 53")
+	}
 	minSize := c.MinSize
 	maxSize := c.MaxSize
 	buf := c.buf
@@ -291,25 +296,23 @@ func (c *Chunker) Next(data []byte) (Chunk, error) {
 		wpos := c.wpos
 		for _, b := range buf[c.bpos:c.bmax] {
 			// slide(b)
+			// limit wpos before to elide array bound checks
+			wpos = wpos % windowSize
 			out := win[wpos]
 			win[wpos] = b
-			digest ^= uint64(tabout[out])
-			wpos = (wpos + 1) % windowSize
+			digest ^= uint64(tab.out[out])
+			wpos++
 
-			// updateDigest
-			index := byte(digest >> polShift)
-			digest <<= 8
-			digest |= uint64(b)
-
-			digest ^= uint64(tabmod[index])
+			digest = updateDigest(digest, polShift, tab, b)
 			// end manual inline
 
 			add++
-			if add < minSize {
-				continue
-			}
 
 			if (digest&c.splitmask) == 0 || add >= maxSize {
+				if add < minSize {
+					continue
+				}
+
 				i := add - c.count - 1
 				data = append(data, c.buf[c.bpos:c.bpos+uint(i)+1]...)
 				c.count = add
@@ -331,7 +334,7 @@ func (c *Chunker) Next(data []byte) (Chunk, error) {
 		}
 		c.digest = digest
 		c.window = win
-		c.wpos = wpos
+		c.wpos = wpos % windowSize
 
 		steps := c.bmax - c.bpos
 		if steps > 0 {
@@ -343,7 +346,7 @@ func (c *Chunker) Next(data []byte) (Chunk, error) {
 	}
 }
 
-func updateDigest(digest uint64, polShift uint, tab tables, b byte) (newDigest uint64) {
+func updateDigest(digest uint64, polShift uint, tab *tables, b byte) (newDigest uint64) {
 	index := digest >> polShift
 	digest <<= 8
 	digest |= uint64(b)
@@ -358,7 +361,7 @@ func (c *Chunker) slide(digest uint64, b byte) (newDigest uint64) {
 	digest ^= uint64(c.tables.out[out])
 	c.wpos = (c.wpos + 1) % windowSize
 
-	digest = updateDigest(digest, c.polShift, c.tables, b)
+	digest = updateDigest(digest, c.polShift, &c.tables, b)
 	return digest
 }
 
